/*******************************************************************************
 *******************************************************************************
 **                                                                           **
 **                                                                           **
 **  Copyright 2003-2006 Tom Bascom, Greenfield Technologies                  **
 **  http://www.greenfieldtech.com                                            **
 **                                                                           **
 **  ProTop is free software; you can redistribute it and/or modify it        **
 **  under the terms of the GNU General Public License (GPL) as published     **
 **  by the Free Software Foundation; either version 2 of the License, or     **
 **  at your option) any later version.                                       **
 **                                                                           **
 **  ProTop is distributed in the hope that it will be useful, but WITHOUT    **
 **  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    **
 **  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     **
 **  for more details.                                                        **
 **                                                                           **
 **  See TERMS.TXT for more information regarding the Terms and Conditions    **
 **  of use and alternative licensing options for this software.              **
 **                                                                           **
 **  A copy of the GPL is in GPL.TXT which was provided with this package.    **
 **                                                                           **
 **  See http://www.fsf.org for more information about the GPL.               **
 **                                                                           **
 **                                                                           **
 *******************************************************************************
 *******************************************************************************
 *
 * protop-cfg.p
 *
 *
 * Initialize global shared variables etc /* configured in etc/protop.cfg */
 *
 *
 * Author:
 *
 *	Tom Bascom, Greenfield Technologies
 *	http://www.greenfieldtech.com
 *	June 15, 2006
 *
 * 	Moved responsibility for configuration of log files etc
 *	from lib/protop.i to lib/protop-cfg.p
 *	June 15, 2006
 * 
 */

define variable ptCfg_date as date    no-undo.
define variable ptCfg_time as integer no-undo.

define variable i as integer no-undo.
define variable j as integer no-undo.

{lib/protop.i}

/*** Install self as a session super-procedure
 ***
 ***/

session:add-super-procedure( this-procedure ).

/* only do this once -- at startup
 */

pt_uniqName  = ldbname( 1 ).
if pt_uniqName = ? then pt_uniqName = "pt".

/* if the user is not running with -rand 2 then random() will always repeat -- so we need to salt it
 * this is ugly but it doesn't need to be "truly" random for any other reason, we're just trying to
 * avoid obvious naming collisions
 */

do i = 1 to (( day( today ) * year( today )) / 50 ):
  j = random( 0, day( today ) * month( today )).
end.

pt_uniqName = pt_uniqName + string( random( 0, max( 100, integer( time ))), "99999" ).

if dbgMode >= 5 then
  message now "pt_uniqName = " pt_uniqName.

assign
  pt_logname    = "&5.&2.&3"
  ptCfg_date    = ?
  ptCfg_time    = 0
  pt_votrx      = 1800
  pt_lktbllim   = 100000
  pt_bkupstale  = 26
  pt_bogomips   = 1000000
  pt_ioresp     = 100
  pt_ioFileName = ""
.

{ssg/sausage14.i}

run ptConfig.

if dbgMode >= 5 then
  message now
    "pt_tmpdir  = " pt_tmpdir  " "
    "pt_logdir  = " pt_logdir  " "
    "pt_rptdir  = " pt_rptdir  " "
    "pt_mailcmd = " pt_mailcmd
  .

if dbgMode >= 5 then
  message now "protop-cfg.p complete.".

return.


/*** Make ptConfig available as a callable super-procedure
 ***
 ***/

procedure ptConfig:

  define variable dbListLine as character no-undo.

  define variable i as integer   no-undo.

  define variable k as character no-undo.
  define variable v as character no-undo.

  if connected( "dictdb" ) then run lib/setptname.p.


  /* support variations of logic based on the specific OS
   */

  if opsys begins "WIN" then
    pt_osName = "Windows".
   else
    do:
      input stream inStrm through value( "uname -a" ).
      import stream inStrm pt_osName.
      input stream inStrm close.
    end.


  /* set some reasonable defaults for key config variables
   */

  assign
    pt_mailcmd   = os-getenv( "MAILCMD" )	/*	'mailx -s "&1" ' 	*/
    pt_logdir    = os-getenv( "LOGDIR" )	/*	"/tmp"			*/
    pt_tmpdir    = os-getenv( "PTTMP" )		/*	TMPDIR gets whacked by the C library :(		*/
    pt_rptdir    = os-getenv( "RPTDIR" )	/*	"/tmp"			*/
  .

  if pt_mailcmd = ? or pt_mailcmd = "" then
    do:
      if opsys = "unix" then
        pt_mailcmd = 'mailx &3 -s "&2" &1'.
       else
        pt_mailcmd = 'mailx &1 "&2" '.
    end.

  if pt_logdir = ? or pt_logdir = "" then
    do:

      file-info:file-name = search( "logs" ).
      if file-info:full-pathname = ? then file-info:file-name = search( "logdir" ).
      if file-info:full-pathname = ? then file-info:file-name = "./logs".
      if file-info:full-pathname = ? then file-info:file-name = "./logdir".
      if file-info:full-pathname = ? then file-info:file-name = "./tmp".
      if file-info:full-pathname = ? then file-info:file-name = "./temp".

      if file-info:full-pathname <> ? then
        pt_logdir  = file-info:full-pathname.
       else
        pt_logdir  = session:temp-directory.

    end.

  if pt_tmpdir = ? or pt_tmpdir = "" then
    do:

      file-info:file-name = search( "tmp" ).
      if file-info:full-pathname = ? then file-info:file-name = "./tmp".

      if file-info:full-pathname <> ? then
        pt_tmpdir  = file-info:full-pathname.
       else
        pt_tmpdir  = session:temp-directory.

    end.

  /* figure out the server name
   *
   * the environment variable is useful in a cluster to set the cluster name rather than the node name
   *
   */

  pt_server = trim( os-getenv( "PTSERVER" )).

  if pt_server = ? or pt_server = "" then
    pt_server = getHostName().

  file-info:file-name = search( "etc/dblist.cfg" ).
  
  if file-info:full-pathname <> ? then
    do:

      input stream inStrm from value( file-info:full-pathname ).
      repeat:

        dbListLine = "".
        import stream inStrm unformatted dbListLine.

        if dbListLine = "" or dbListLine begins "#" then next.
        if entry( 1, dbListLine, "|" ) <> pt_shortName then next.
        if num-entries( dbListLine, "|" ) < 6 then next.

        pt_resrcType = entry( 6, dbListLine, "|" ).

      end.
      input stream inStrm close.

    end.

  assign
    pt_logname    = "&5.&2.&3"
    ptCfg_date    = ?
    ptCfg_time    = 0
    pt_votrx      = 1800
    pt_lktbllim   = 100000
    pt_bkupstale  = 26
    pt_bogomips   = 1000000
    pt_ioresp     = 100
    pt_ioFileName = ""
    pt_AICheckInterval   = 60
    pt_PICACheckInterval = 60
    pt_appsrvStuck = 120
    pt_zoomTo      = 30
  .

  pt_sampleLimit = integer( os-getenv( "SAMPLELIMIT" )) no-error.
  if pt_sampleLimit = ? then pt_sampleLimit = -1.

  {ssg/sausage14.i}

  /* bin/localenv variables
   */

  if os-getenv( "IOFILENAME" ) <> ? then pt_ioFileName = os-getenv( "IOFILENAME" ).
  if os-getenv( "DFCMD" )      <> ? then pt_dfCmd      = os-getenv( "DFCMD" ).
  if os-getenv( "DFCMDI" )     <> ? then pt_dfCmdI     = os-getenv( "DFCMDI" ).
  if os-getenv( "MNTCMD" )     <> ? then pt_mntCmd     = os-getenv( "MNTCMD" ).
  if os-getenv( "VOTRX" )      <> ? then pt_votrx      = integer( os-getenv( "VOTRX"    )) no-error.
  if os-getenv( "LKTBLLIM" )   <> ? then pt_lktbllim   = integer( os-getenv( "LKTBLLIM" )) no-error.
  if os-getenv( "BOGOMIPS" )   <> ? then pt_bogomips   = integer( os-getenv( "BOGOMIPS" )) no-error.
  if os-getenv( "IORESP" )     <> ? then pt_ioresp     = integer( os-getenv( "IORESP"   )) no-error.

  if os-getenv( "AICHECKINTERVAL" )   <> ? then pt_AICheckInterval   = integer( os-getenv( "AICHECKINTERVAL"   )) no-error.
  if os-getenv( "PICACHECKINTERVAL" ) <> ? then pt_PICACheckInterval = integer( os-getenv( "PICACHECKINTERVAL" )) no-error.
  if os-getenv( "APPSRVSTUCK" )       <> ? then pt_appsrvStuck       = integer( os-getenv( "APPSRVSTUCK"       )) no-error.
  if os-getenv( "DBSHUTNAG" )         <> ? then pt_dbShutNag         = integer( os-getenv( "DBSHUTNAG"         )) no-error.

  if os-getenv( "ZOOMTO" )            <> ? then pt_zoomTo            = integer( os-getenv( "ZOOMTO"            )) no-error.
  if pt_zoomTo = ? or pt_zoomTo = 0 then pt_zoomTo = 30.

  /* alert on long bi backup phase
   */

  pt_bibkUpInfo = integer( os-getenv( "BIBKUP_INFO" )) no-error.
  if pt_bibkUpInfo = ? then pt_bibkUpInfo = 0.

  pt_bibkUpAlert = integer( os-getenv( "BIBKUP_ALERT" )) no-error.
  if pt_bibkUpAlert = ? then pt_bibkUpAlert = 60.

  pt_bibkUpAlarm = integer( os-getenv( "BIBKUP_ALARM" )) no-error.
  if pt_bibkUpAlarm = ? then pt_bibkUpAlarm = 300.

  pt_bibkUpPage  = integer( os-getenv( "BIBKUP_PAGE" )) no-error.
  if pt_bibkUpPage = ? then pt_bibkUpPage = 0.

  /* alert on long backups
   */

  pt_bkUpInfo = integer( os-getenv( "BKUP_INFO" )) no-error.
  if pt_bkUpInfo = ? then pt_bkUpInfo = 0.

  pt_bkUpAlert = integer( os-getenv( "BKUP_ALERT" )) no-error.
  if pt_bkUpAlert = ? then pt_bkUpAlert = 3600.

  pt_bkUpAlarm = integer( os-getenv( "BKUP_ALARM" )) no-error.
  if pt_bkUpAlarm = ? then pt_bkUpAlarm = 14400.

  pt_bkUpPage  = integer( os-getenv( "BKUP_PAGE" )) no-error.
  if pt_bkUpPage = ? then pt_bkUpPage = 0.

  /* alert on long quiet points
   */

  pt_qpInfo = integer( os-getenv( "QP_INFO" )) no-error.
  if pt_qpInfo = ? then pt_qpInfo = 60.

  pt_qpAlert = integer( os-getenv( "QP_ALERT" )) no-error.
  if pt_qpAlert = ? then pt_qpAlert = 60.

  pt_qpAlarm = integer( os-getenv( "QP_ALARM" )) no-error.
  if pt_qpAlarm = ? then pt_qpAlarm = 300.

  pt_qpPage  = integer( os-getenv( "QP_PAGE" )) no-error.
  if pt_qpPage = ? then pt_qpPage = 600.


  /* alert on long ai switch (extent copy)
   */

  pt_aiswInfo = integer( os-getenv( "AISW_INFO" )) no-error.
  if pt_aiswInfo = ? then pt_aiswInfo = 0.

  pt_aiswAlert = integer( os-getenv( "AISW_ALERT" )) no-error.
  if pt_aiswAlert = ? then pt_aiswAlert = 20.

  pt_aiswAlarm = integer( os-getenv( "AISW_ALARM" )) no-error.
  if pt_aiswAlarm = ? then pt_aiswAlarm = 60.

  pt_aiswPage = integer( os-getenv( "AISW_PAGE" )) no-error.
  if pt_aiswPage = ? then pt_aiswPage = 0.

  pt_helplink = os-getenv( "PTHELPLINK" ) no-error.
  if pt_helplink = ? then pt_helplink = "https://wss.com/protop/help/".

  pt_commlink = os-getenv( "PTCOMMLINK" ) no-error.
  if pt_commlink = ? then pt_commlink = "https://community.wss.com/".


  /* pro2 behaviour
   */

  pt_pro2qCountLim = int64( os-getenv( "PRO2QCOUNTLIM" )) no-error.
  if pt_pro2qCountLim = ? then pt_pro2qCountLim = 1000.

  pt_pro2qSkip2Seq = int64( os-getenv( "PRO2QSKIP2SEQ" )) no-error.
  if pt_pro2qSkip2Seq = ? then pt_pro2qSkip2Seq = 0.

  if os-getenv( "PRO2QESTIMATE" ) begins "n" then
    pt_pro2qEstimate = no.
   else
    pt_pro2qEstimate = yes.


  /* related to getting process info via "wmic" (windows) or "ps" (unix)
   */

  if integer( entry( 1, proversion, "." )) < 11 then
    pt_usewmic4ps = yes.
   else
    do:
      pt_usewmic4ps = no.
      if os-getenv( "USEWMIC4PS" ) = "yes" then
        pt_usewmic4ps = yes.
    end.

  pt_wmicPIDlbl = os-getenv( "WMICPIDLBL" ).
  if pt_wmicPIDlbl = ? then pt_wmicPIDlbl = "ProcessId".

  pt_getPID = os-getenv( "GETPID" ).
  if pt_getPID = ? then
    do:
      if pt_osName = "HP-UX" then
        pt_getPID = "ps -p &1 -fx".	/* the "x" returns long arglists	*/
       else
        pt_getPID = "ps -p &1 -f".
    end.

  /* timing for 1,600 or so _progres processes
   * 
   * $ time ps -C _progres -f | wc -l
   * 1681
   * 
   * real    0m1.217s
   * user    0m0.333s
   * sys     0m0.632s
   * 
   * $ time ps -ef | grep [_]progres | wc -l
   * 1681
   * 
   * real    0m0.979s
   * user    0m0.381s
   * sys     0m0.517s
   * 
   * in _theory_ it seems like there could be optimizations inside "ps" to make selecting by command faster
   * in practice it doesn't really seem to help
   * likewise, the sequence of grep filters in a more comple pipeline doesn't seem to have a material impact
   * 
   */

  pt_getAgentList = os-getenv( "GETAGENTLIST" ).
  if pt_getAgentList = ? then
    do:
      if pt_osName = "HP-UX" then
        pt_getAgentList = 'ps -C _progres -fx | grep "p util/pt3agent"'.
       else
        pt_getAgentList = 'ps -C _progres -f  | grep "p util/pt3agent"'.
    end.


  /* other tweakable features
   */

  if os-getenv( "USEREXP" ) begins "n" then
    pt_doZippy = no.
   else
    pt_doZippy = yes.

  if os-getenv( "USERFUTIL" ) begins "n" then
    pt_useRFUtil = no.
   else
    pt_useRFUtil = yes.

  if os-getenv( "USERLOCK" ) begins "y" then
    pt_userLock = yes.
   else
    do:
      if connected( "dictdb"  ) then run lib/userlock.p ( input-output pt_userLock ).
    end.

  if os-getenv( "DBIDOWNGRADE" ) begins "n" then
    pt_DBIDowngrade = no.
   else
    pt_DBIDowngrade = yes.

  pt_updAreaData = integer( os-getenv( "UPDAREADATA" )) no-error.
  if pt_updAreaData = ? then pt_updAreaData = 3600 * 6.

  if os-getenv( "USEACTBUFF" ) begins "n" then
    pt_actBuff = no.
   else
    pt_actBuff = yes.


  if pt_DynLIBC = ? then							/* not set, check for an environment variable				*/
    do:
      if os-getenv( "DYNLIBC" ) = "no" then
        pt_DynLIBC = no.							/* explictly avoid trying to use libc					*/
    end.									/* if not "no" it isn't automatically yes				*/

  if pt_DynLIBC = ? then
    pt_DynLIBC = ( integer( entry( 1, proversion, "." )) >= 11 ).		/* oe11+ are ok								*/

  if pt_osName = "AIX" then							/* AIX doesn't have an easy to find libc.so (libc.a is present)		*/
    pt_DynLIBC = no.


  if os-getenv( "AUTORESIZE" ) begins "n" then
    pt_autoresize = no.
   else
    do:
      if pt_osname = "Linux" then
        pt_autoresize = yes.
       else
        pt_autoresize = no.
    end.



  /* restricted features
   */

  file-info:file-name = "ssg/sausage26.r".
  if file-info:full-pathname = ? then
    file-info:file-name = "ssg/sausage26.p".

  if file-info:full-pathname <> ? then
    run ssg/sausage26.p.

end.
